@@ -4,12 +4,13 @@ use strict;
 use warnings;
 use Carp ();
 use IO::File ();
-use IPC::Run ();
+use IPC::Open2 ();
 
 use PDF::WebKit::Configuration;
 use PDF::WebKit::Source;
 
-our $VERSION = '0.10';
+# should roughly coincide with PDFKit versions
+our $VERSION = 0.5;
 
 use Moose;
 
@@ -34,12 +35,14 @@ sub BUILD {
   my ($self,$args) = @_;
 
   $self->source( PDF::WebKit::Source->new($args->{source}) );
+
   $self->stylesheets( [] );
-  $self->_set_options({
-    $self->_normalize_options(%{ $self->configuration->default_options }),
-    $self->_normalize_options(%{ $args->{options} }),
-    $self->_normalize_options($self->_find_options_in_meta),
-  });
+
+  my %options;
+  %options = ( %{ $self->configuration->default_options }, %{ $args->{options} } );
+  %options = ( %options, $self->_find_options_in_meta($self->source) ) unless $self->source->is_url;
+  %options = $self->_normalize_options(%options);
+  $self->_set_options(\%options);
 
   if (not -e $self->configuration->wkhtmltopdf) {
     my $msg = "No wkhtmltopdf executable found\n";
@@ -60,7 +63,9 @@ sub configure {
 sub command {
   my $self = shift;
   my $path = shift;
-  my @args = ( $self->_executable, $self->_prepare_options, '--quiet' );
+  my @args = ( $self->_executable );
+  push @args, %{ $self->options };
+  push @args, '--quiet';
   
   if ($self->source->is_html) {
     push @args, '-';  # Get HTML from stdin
@@ -71,7 +76,7 @@ sub command {
 
   push @args, $path || '-'; # write to file or stdout
 
-  return grep { defined($_) } @args;
+  return map { s/"/\\"/g; qq{"$_"} } grep { defined($_) } @args;
 }
 
 sub _executable {
@@ -92,20 +97,22 @@ sub to_pdf {
 
   $self->_append_stylesheets;
   my @args = $self->command($path);
-
-  my $input = $self->source->is_html ? $self->source->content : undef;
-  my $output;
-
-  IPC::Run::run( \@args, "<", \$input, ">", \$output );
-
+  my ($PDF_OUT,$PDF_IN);
+  eval { IPC::Open2::open2($PDF_OUT, $PDF_IN, join(" ", @args)) };
+  if ($@) {
+    die "can't execute $args[0]: $!";
+  }
+  print {$PDF_IN} $self->source->content if $self->source->is_html;
+  close($PDF_IN) || die $!;
+  my $result = do { local $/; <$PDF_OUT> };
   if ($path) {
-    $output = do { local (@ARGV,$/) = ($path); <> };
+    $result = do { local (@ARGV,$/) = ($path); <> };
   }
 
-  if (not (defined($output) && length($output))) {
+  if (not (defined($result) && length($result))) {
     Carp::croak "command failed: $args[0]";
   }
-  return $output;
+  return $result;
 }
 
 sub to_file {
@@ -119,32 +126,25 @@ sub to_file {
 }
 
 sub _find_options_in_meta {
-  my ($self) = @_;
-  return () if $self->source->is_url;
+  my ($self,$source) = @_;
   # if we can't parse for whatever reason, keep calm and carry on.
-  my @result = eval { $self->_pdf_webkit_meta_tags };
+  my @result = eval { $self->_pdf_webkit_meta_tags($source) };
   return $@ ? () : @result;
 }
 
 sub _pdf_webkit_meta_tags {
-  my ($self) = @_;
+  my ($self,$source) = @_;
   return () unless eval { require XML::LibXML };
-  my $source = $self->source;
 
   my $prefix = $self->configuration->meta_tag_prefix;
-
-  # these options do not work at the constructor level in XML::LibXML 1.70, so pass
-  # them through to the parser.
-  my %options = (
+  my $parser = XML::LibXML->new(
     recover => 2,
     suppress_errors => 1,
     suppress_warnings => 1,
     no_network => 1,
   );
-
-  my $parser = XML::LibXML->new();
-  my $doc = $source->is_html ? $parser->parse_html_string($source->content,\%options)
-          : $source->is_file ? $parser->parse_html_file($source->string,\%options)
+  my $doc = $source->is_html ? $parser->parse_html_string($source->content)
+          : $source->is_file ? $parser->parse_html_file($source->string)
           : return ();
 
   my %meta;
@@ -182,29 +182,14 @@ sub _append_stylesheets {
   $self->source->string(\$html);
 }
 
-sub _prepare_options {
-  my ($self) = @_;
-  my $options = $self->options;
-  my @args;
-  while (my ($name,$val) = each %$options) {
-    next unless defined($val) && length($val);
-    if ($val eq 'yes' || $val eq 'YES') {
-      push @args, $name;
-    }
-    else {
-      push @args, $name, $val;
-    }
-  }
-  return @args;
-}
-
 sub _normalize_options {
   my $self = shift;
   my %orig_options = @_;
   my %normalized_options;
   while (my ($key,$val) = each %orig_options) {
-    my $normalized_key = $self->_normalize_arg($key);
-    $normalized_options{$normalized_key} = $val;
+    next unless defined($val) && length($val);
+    my $normalized_key = "--" . $self->_normalize_arg($key);
+    $normalized_options{$normalized_key} = $self->_normalize_value($val);
   }
   return %normalized_options;
 }
@@ -213,10 +198,19 @@ sub _normalize_arg {
   my ($self,$arg) = @_;
   $arg =~ lc($arg);
   $arg =~ s{[^a-z0-9]}{-}g;
-  $arg =~ s{^-*}{--};
   return $arg;
 }
 
+sub _normalize_value {
+  my ($self,$value) = @_;
+  if (defined($value) && ($value eq 'yes' || $value eq 'YES')) {
+    return undef;
+  }
+  else {
+    return $value;
+  }
+}
+
 no Moose;
 __PACKAGE__->meta->make_immutable;
 
